@@ -4,17 +4,19 @@ from gettext import gettext as _
 from gi.repository import GLib, GObject, Gtk, Pango
 
 from lutris import runners, services
-from lutris.database import categories as categories_db
 from lutris.database import games as games_db
+from lutris.exceptions import watch_errors
 from lutris.game import Game
+from lutris.gui import dialogs
 from lutris.gui.config.runner import RunnerConfigDialog
 from lutris.gui.config.runner_box import RunnerBox
 from lutris.gui.config.services_box import ServicesBox
 from lutris.gui.dialogs import ErrorDialog
 from lutris.gui.dialogs.runner_install import RunnerInstallDialog
 from lutris.gui.widgets.utils import has_stock_icon
+from lutris.installer.interpreter import ScriptInterpreter
+from lutris.services import SERVICES
 from lutris.services.base import AuthTokenExpired, BaseService
-from lutris.util.jobs import AsyncCall
 
 TYPE = 0
 SLUG = 1
@@ -22,6 +24,8 @@ ICON = 2
 LABEL = 3
 GAMECOUNT = 4
 
+SERVICE_INDICES = {name: index for index, name in enumerate(SERVICES.keys())}
+
 
 class SidebarRow(Gtk.ListBoxRow):
     """A row in the sidebar containing possible action buttons"""
@@ -67,6 +71,12 @@ class SidebarRow(Gtk.ListBoxRow):
         self.spinner = Gtk.Spinner()
         self.box.pack_end(self.spinner, False, False, 0)
 
+    @property
+    def sort_key(self):
+        """An index indicate the place this row has within its type. The id is used
+        as a tie-breaker."""
+        return 0
+
     def get_actions(self):
         return []
 
@@ -118,15 +128,25 @@ class ServiceSidebarRow(SidebarRow):
             service.id,
             "service",
             service.name,
-            Gtk.Image.new_from_icon_name(service.icon, Gtk.IconSize.MENU)
+            LutrisSidebar.get_sidebar_icon(service.icon)
         )
         self.service = service
 
+    @property
+    def sort_key(self):
+        return SERVICE_INDICES[self.id]
+
     def get_actions(self):
         """Return the definition of buttons to be added to the row"""
-        return [
+        displayed_buttons = []
+        if self.service.is_launchable():
+            displayed_buttons.append(
+                ("media-playback-start-symbolic", _("Run"), self.on_service_run, "run")
+            )
+        displayed_buttons.append(
             ("view-refresh-symbolic", _("Reload"), self.on_refresh_clicked, "refresh")
-        ]
+        )
+        return displayed_buttons
 
     def on_service_run(self, button):
         """Run a launcher associated with a service"""
@@ -138,15 +158,15 @@ class ServiceSidebarRow(SidebarRow):
         if self.service.online and not self.service.is_connected():
             self.service.logout()
             return
-        AsyncCall(self.service.reload, self.service_load_cb)
+        self.service.start_reload(self.service_reloaded_cb)
 
-    def service_load_cb(self, _result, error):
+    def service_reloaded_cb(self, error):
         if error:
             if isinstance(error, AuthTokenExpired):
                 self.service.logout()
-                self.service.login()
+                self.service.login(parent=self.get_toplevel())
             else:
-                ErrorDialog(str(error))
+                ErrorDialog(str(error), parent=self.get_toplevel())
         GLib.timeout_add(2000, self.enable_refresh_button)
 
     def enable_refresh_button(self):
@@ -179,7 +199,7 @@ class OnlineServiceSidebarRow(ServiceSidebarRow):
         if self.service.is_authenticated():
             self.service.logout()
         else:
-            self.service.login()
+            self.service.login(parent=self.get_toplevel())
         self.create_button_box()
 
 
@@ -201,25 +221,38 @@ class RunnerSidebarRow(SidebarRow):
                 "manage-versions"
             ))
         if self.runner.runnable_alone:
-            entries.append(("media-playback-start-symbolic", _("Run"), self.runner.run, "run"))
+            entries.append(("media-playback-start-symbolic", _("Run"), self.on_run_runner, "run"))
         entries.append(("emblem-system-symbolic", _("Configure"), self.on_configure_runner, "configure"))
         return entries
 
+    @watch_errors()
+    def on_run_runner(self, *_args):
+        """Runs the runner without no game."""
+        self.runner.run(self.get_toplevel())
+
+    @watch_errors()
     def on_configure_runner(self, *_args):
         """Show runner configuration"""
         self.application.show_window(RunnerConfigDialog, runner=self.runner)
 
+    @watch_errors()
     def on_manage_versions(self, *_args):
         """Manage runner versions"""
         dlg_title = _("Manage %s versions") % self.runner.name
-        RunnerInstallDialog(dlg_title, self.get_toplevel(), self.runner.name)
+        self.application.show_window(RunnerInstallDialog, title=dlg_title,
+                                     runner=self.runner, parent=self.get_toplevel())
+
+    def on_watched_error(self, error):
+        dialogs.ErrorDialog(str(error), parent=self.get_toplevel())
 
 
 class SidebarHeader(Gtk.Box):
     """Header shown on top of each sidebar section"""
 
-    def __init__(self, name):
+    def __init__(self, name, header_index):
         super().__init__(orientation=Gtk.Orientation.VERTICAL)
+        self.header_index = header_index
+        self.first_row = None
         self.get_style_context().add_class("sidebar-header")
         label = Gtk.Label(
             halign=Gtk.Align.START,
@@ -248,36 +281,38 @@ class DummyRow():
 class LutrisSidebar(Gtk.ListBox):
     __gtype_name__ = "LutrisSidebar"
 
-    def __init__(self, application, selected=None):
+    def __init__(self, application):
         super().__init__()
         self.set_size_request(200, -1)
         self.application = application
         self.get_style_context().add_class("sidebar")
+
+        # Empty values until LutrisWindow explicitly initializes the rows
+        # at the right time.
         self.installed_runners = []
+        self.active_services = {}
+        self.active_platforms = []
         self.service_rows = {}
-        self.active_platforms = None
-        self.runners = None
-        self.platforms = None
-        self.categories = None
+        self.runner_rows = {}
+        self.platform_rows = {}
+
         # A dummy objects that allows inspecting why/when we have a show() call on the object.
         self.running_row = DummyRow()
-        if selected:
-            self.selected_row_type, self.selected_row_id = selected.split(":")
-        else:
-            self.selected_row_type, self.selected_row_id = ("category", "all")
+        self.missing_row = DummyRow()
         self.row_headers = {
-            "library": SidebarHeader(_("Library")),
-            "sources": SidebarHeader(_("Sources")),
-            "runners": SidebarHeader(_("Runners")),
-            "platforms": SidebarHeader(_("Platforms")),
+            "library": SidebarHeader(_("Library"), header_index=0),
+            "service": SidebarHeader(_("Sources"), header_index=1),
+            "runner": SidebarHeader(_("Runners"), header_index=2),
+            "platform": SidebarHeader(_("Platforms"), header_index=3),
         }
-        GObject.add_emission_hook(RunnerBox, "runner-installed", self.update)
-        GObject.add_emission_hook(RunnerBox, "runner-removed", self.update)
-        GObject.add_emission_hook(ServicesBox, "services-changed", self.on_services_changed)
+        GObject.add_emission_hook(RunnerBox, "runner-installed", self.update_rows)
+        GObject.add_emission_hook(RunnerBox, "runner-removed", self.update_rows)
+        GObject.add_emission_hook(ScriptInterpreter, "runners-installed", self.update_rows)
+        GObject.add_emission_hook(ServicesBox, "services-changed", self.update_rows)
         GObject.add_emission_hook(Game, "game-start", self.on_game_start)
         GObject.add_emission_hook(Game, "game-stop", self.on_game_stop)
-        GObject.add_emission_hook(Game, "game-updated", self.update)
-        GObject.add_emission_hook(Game, "game-removed", self.update)
+        GObject.add_emission_hook(Game, "game-updated", self.update_rows)
+        GObject.add_emission_hook(Game, "game-removed", self.update_rows)
         GObject.add_emission_hook(BaseService, "service-login", self.on_service_auth_changed)
         GObject.add_emission_hook(BaseService, "service-logout", self.on_service_auth_changed)
         GObject.add_emission_hook(BaseService, "service-games-load", self.on_service_games_updating)
@@ -286,7 +321,8 @@ class LutrisSidebar(Gtk.ListBox):
         self.set_header_func(self._header_func)
         self.show_all()
 
-    def get_sidebar_icon(self, icon_name):
+    @staticmethod
+    def get_sidebar_icon(icon_name):
         name = icon_name if has_stock_icon(icon_name) else "package-x-generic-symbolic"
         icon = Gtk.Image.new_from_icon_name(name, Gtk.IconSize.MENU)
 
@@ -300,14 +336,12 @@ class LutrisSidebar(Gtk.ListBox):
 
     def initialize_rows(self):
         """
-        Select the initial row; this triggers the initialization of the game view
+        Select the initial row; this triggers the initialization of the game view,
         so we must do this even if this sidebar is never realized, but only after
         the sidebar's signals are connected.
         """
-        self.active_platforms = games_db.get_used_platforms()
-        self.runners = sorted(runners.__all__)
-        self.platforms = sorted(runners.RUNNER_PLATFORMS)
-        self.categories = categories_db.get_categories()
+
+        # Create the basic rows that are not data dependant
 
         self.add(
             SidebarRow(
@@ -335,6 +369,13 @@ class LutrisSidebar(Gtk.ListBox):
                 Gtk.Image.new_from_icon_name("favorite-symbolic", Gtk.IconSize.MENU)
             )
         )
+        self.missing_row = SidebarRow(
+            "missing",
+            "dynamic_category",
+            _("Missing"),
+            Gtk.Image.new_from_icon_name("dialog-warning-symbolic", Gtk.IconSize.MENU)
+        )
+        self.add(self.missing_row)
 
         self.running_row = SidebarRow(
             "running",
@@ -344,43 +385,34 @@ class LutrisSidebar(Gtk.ListBox):
         )
         # I wanted this to be on top but it really messes with the headers when showing/hiding the row.
         self.add(self.running_row)
-
-        service_classes = services.get_enabled_services()
-        for service_name in service_classes:
-            service = service_classes[service_name]()
-            row_class = OnlineServiceSidebarRow if service.online else ServiceSidebarRow
-            service_row = row_class(service)
-            self.service_rows[service_name] = service_row
-            self.add(service_row)
-
-        for runner_name in self.runners:
-            icon_name = runner_name.lower().replace(" ", "") + "-symbolic"
-            runner = runners.import_runner(runner_name)()
-            self.add(RunnerSidebarRow(
-                runner_name,
-                "runner",
-                runner.human_name,
-                self.get_sidebar_icon(icon_name),
-                application=self.application
-            ))
-
-        for platform in self.platforms:
-            icon_name = (platform.lower().replace(" ", "").replace("/", "_") + "-symbolic")
-            self.add(SidebarRow(platform, "platform", platform, self.get_sidebar_icon(icon_name)))
-
-        self.update()
-
-        for row in self.get_children():
-            if row.type == self.selected_row_type and row.id == self.selected_row_id:
-                self.select_row(row)
-                break
-
         self.show_all()
         self.running_row.hide()
 
+        # Create the dynamic rows that are initially needed
+        self.update_rows()
+
+    @property
+    def selected_category(self):
+        """The selected sidebar row, as a tuple of category and category value,
+        like ('service', 'lutris')."""
+        row = self.get_selected_row()
+        return (row.type, row.id) if row else ("category", "all")
+
+    @selected_category.setter
+    def selected_category(self, value):
+        """Selects the rrow for the category indicated by a category tuple,
+        like ('service', 'lutris')"""
+        selected_row_type, selected_row_id = value or ("category", "all")
+        for row in self.get_children():
+            if row.type == selected_row_type and row.id == selected_row_id:
+                self.select_row(row)
+                break
+
     def _filter_func(self, row):
-        if not row or not row.id or row.type in ("category", "dynamic_category", "service"):
+        if not row or not row.id or row.type in ("category", "dynamic_category"):
             return True
+        if row.type == "service":
+            return row.id in self.active_services
         if row.type == "runner":
             if row.id is None:
                 return True  # 'All'
@@ -389,19 +421,91 @@ class LutrisSidebar(Gtk.ListBox):
 
     def _header_func(self, row, before):
         if not before:
-            row.set_header(self.row_headers["library"])
+            header = self.row_headers["library"]
         elif before.type in ("category", "dynamic_category") and row.type == "service":
-            row.set_header(self.row_headers["sources"])
+            header = self.row_headers[row.type]
         elif before.type == "service" and row.type == "runner":
-            row.set_header(self.row_headers["runners"])
+            header = self.row_headers[row.type]
         elif before.type == "runner" and row.type == "platform":
-            row.set_header(self.row_headers["platforms"])
+            header = self.row_headers[row.type]
         else:
-            row.set_header(None)
+            header = None
 
-    def update(self, *_args):
+        if row.get_header() != header:
+            # GTK is messy here; a header can't belong to two rows at once,
+            # so we must remove it from the one that owns it, if any, and
+            # also from the sidebar itself. Then we can reuse it.
+            if header.first_row:
+                header.first_row.set_header(None)
+                if header.get_parent() == self:
+                    self.remove(header)
+            header.first_row = row
+            row.set_header(header)
+
+    def update_rows(self, *_args):
+        """Generates any missing rows that are now needed, and re-evaluate the filter to hide
+        any no longer needed. GTK has a lot of trouble dynamically updating and re-arranging
+        rows, so this will have to do. This keeps the total row count down reasonably well."""
+
+        def get_sort_key(row):
+            """Returns a key used to sort the rows. This keeps rows for a header
+            together, and rows in a hopefully reasonable order as we insert them."""
+            header_row = self.row_headers.get(row.type) if row.type else None
+            header_index = header_row.header_index if header_row else 0
+            return header_index, row.sort_key, row.id
+
+        def insert_row(row):
+            """Find the best place to insert the row, to maintain order, and inserts it there."""
+            index = 0
+            seq = get_sort_key(row)
+            while True:
+                r = self.get_row_at_index(index)
+                if not r or get_sort_key(r) > seq:
+                    break
+                index += 1
+
+            row.show_all()
+            self.insert(row, index)
+
+        self.active_services = services.get_enabled_services()
         self.installed_runners = [runner.name for runner in runners.get_installed()]
         self.active_platforms = games_db.get_used_platforms()
+
+        for service_name, service_class in self.active_services.items():
+            if service_name not in self.service_rows:
+                service = service_class()
+                row_class = OnlineServiceSidebarRow if service.online else ServiceSidebarRow
+                service_row = row_class(service)
+                self.service_rows[service_name] = service_row
+                insert_row(service_row)
+
+        for runner_name in self.installed_runners:
+            if runner_name not in self.runner_rows:
+                icon_name = runner_name.lower().replace(" ", "") + "-symbolic"
+                runner = runners.import_runner(runner_name)()
+                runner_row = RunnerSidebarRow(
+                    runner_name,
+                    "runner",
+                    runner.human_name,
+                    self.get_sidebar_icon(icon_name),
+                    application=self.application
+                )
+                self.runner_rows[runner_name] = runner_row
+                insert_row(runner_row)
+
+        for platform in self.active_platforms:
+            if platform not in self.platform_rows:
+                icon_name = platform.lower().replace(" ", "").replace("/", "_") + "-symbolic"
+                platform_row = SidebarRow(
+                    platform,
+                    "platform",
+                    platform,
+                    self.get_sidebar_icon(icon_name),
+                    application=self.application
+                )
+                self.platform_rows[platform] = platform_row
+                insert_row(platform_row)
+
         self.invalidate_filter()
         return True
 
@@ -421,22 +525,19 @@ class LutrisSidebar(Gtk.ListBox):
         return True
 
     def on_service_auth_changed(self, service):
-        self.service_rows[service.id].create_button_box()
-        self.service_rows[service.id].update_buttons()
+        if service.id in self.service_rows:
+            self.service_rows[service.id].create_button_box()
+            self.service_rows[service.id].update_buttons()
         return True
 
     def on_service_games_updating(self, service):
-        self.service_rows[service.id].is_updating = True
-        self.service_rows[service.id].update_buttons()
+        if service.id in self.service_rows:
+            self.service_rows[service.id].is_updating = True
+            self.service_rows[service.id].update_buttons()
         return True
 
     def on_service_games_updated(self, service):
-        self.service_rows[service.id].is_updating = False
-        self.service_rows[service.id].update_buttons()
-        return True
-
-    def on_services_changed(self, _widget):
-        for child in self.get_children():
-            child.destroy()
-        self.initialize_rows()
+        if service.id in self.service_rows:
+            self.service_rows[service.id].is_updating = False
+            self.service_rows[service.id].update_buttons()
         return True
